/*
*  RAL -- Rubrica Addressbook Library
*  file: address.h
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_ADDRESS_H
#define _RUBRICA_ADDRESS_H

#include <glib.h> 
#include <glib-object.h>


#define RUBRICA_ADDRESS_TYPE         (rubrica_address_get_type())

#define RUBRICA_ADDRESS(obj)         (G_TYPE_CHECK_INSTANCE_CAST((obj),   \
                                      RUBRICA_ADDRESS_TYPE,               \
                                      RubricaAddress))

#define RUBRICA_ADDRESS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),    \
                                      RUBRICA_ADDRESS_TYPE,               \
                                      RubricaAddressClass))

#define IS_RUBRICA_ADDRESS(obj)      (G_TYPE_CHECK_INSTANCE_TYPE((obj),   \
                                      RUBRICA_ADDRESS_TYPE))

#define IS_RUBRICA_ADDRESS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), \
                                         RUBRICA_ADDRESS_TYPE))

#define RUBRICA_ADDRESS_GET_CLASS(kls) (G_TYPE_INSTANCE_GET_CLASS((kls),  \
                                        RUBRICA_ADDRESS_TYPE,             \
                                        RubricaAddressClass))


typedef enum {
  RUBRICA_ADDRESS_PREF = 0,        
  RUBRICA_ADDRESS_HOME,        
  RUBRICA_ADDRESS_WORK,        
  RUBRICA_ADDRESS_PARCEL,
  RUBRICA_ADDRESS_POSTAL,
  RUBRICA_ADDRESS_INTL,
  RUBRICA_ADDRESS_DOM,
  RUBRICA_ADDRESS_COMPANY,
  RUBRICA_ADDRESS_UNKNOWN,
  RUBRICA_ADDRESS_INVALID
} RubricaAddressType;



typedef struct _RubricaAddress        RubricaAddress;
typedef struct _RubricaAddressClass   RubricaAddressClass;
typedef struct _RubricaAddressPrivate RubricaAddressPrivate;


struct _RubricaAddress
{
  GObject parent;

  RubricaAddressPrivate* private;
};

struct _RubricaAddressClass
{
  GObjectClass parent_class;
};


GType           rubrica_address_get_type    (void);
 

RubricaAddress* rubrica_address_new         (void);
void            rubrica_address_free        (RubricaAddress* address);

RubricaAddress* rubrica_address_copy        (RubricaAddress* address);

gchar*             rubrica_address_decode_type (RubricaAddressType type);
RubricaAddressType rubrica_address_encode_type (gchar* type);

#endif
