/*
*  RAL -- Rubrica Addressbook Library
*  file: card.h
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_CARD_H
#define _RUBRICA_CARD_H

#include <time.h>
#include <glib.h>
#include <glib-object.h>
#include <libxml/tree.h>

#include "types.h"
#include "group_set.h"
#include "group.h"
#include "address.h"
#include "net.h"
#include "telephone.h"



#define RUBRICA_CARD_TYPE            (rubrica_card_get_type())

#define RUBRICA_CARD(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj),   \
                                      RUBRICA_CARD_TYPE,                  \
                                      RubricaCard))

#define RUBRICA_CARD_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass),    \
                                      RUBRICA_CARD_TYPE,                  \
                                      RubricaCardClass))

#define IS_RUBRICA_CARD(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj),   \
                                      RUBRICA_CARD_TYPE))

#define IS_RUBRICA_CARD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),    \
                                      RUBRICA_CARD_TYPE))

#define RUBRICA_CARD_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj),    \
                                      RUBRICA_CARD_TYPE,                  \
                                      RubricaCardClass))


typedef enum {
  RUBRICA_RANK_NONE = 0,         /* no importance  */
  RUBRICA_RANK_FEW,              /* few importance  */
  RUBRICA_RANK_NORMAL,           /* normal  */
  RUBRICA_RANK_HIGHT,            /* hight importance  */
  RUBRICA_RANK_VERY_HIGHT        /* very hight importance  */
} RubricaRank;


enum {
  UNKNOW_CARD,
  PERSONAL_CARD,
  COMPANY_CARD
};


typedef struct _RubricaCard          RubricaCard;
typedef struct _RubricaCardClass     RubricaCardClass;
typedef struct _RubricaCardPrivate   RubricaCardPrivate;


struct _RubricaCard 
{
  GObject parent;
  
  RubricaCardPrivate* private;
};


struct _RubricaCardClass
{
  GObjectClass parent;
  
  /* methods */
  void     (*free)            (RubricaCard* card);
  gint     (*generate_id)     (RubricaCard* card, gchar* type);
  void     (*align_id)        (RubricaCard* card, gchar* id, gchar* type);

  void     (*write_infos)     (RubricaCard* card, xmlNodePtr node);
  void     (*write_contact)   (RubricaCard* card, xmlNodePtr node);
  void     (*write_group)     (RubricaCard* card, xmlNodePtr node);
  void     (*write_work)      (RubricaCard* card, xmlNodePtr node);  
  void     (*write_addresses) (RubricaCard* card, xmlNodePtr node);
  void     (*write_net)       (RubricaCard* card, xmlNodePtr node);
  void     (*write_telephone) (RubricaCard* card, xmlNodePtr node);  
  void     (*write_notes)     (RubricaCard* card, xmlNodePtr node);

  void     (*read_infos)      (RubricaCard* card, xmlNodePtr node);
  void     (*read_contact)    (RubricaCard* card, xmlNodePtr node);
  void     (*read_group)      (RubricaCard* card, xmlNodePtr node);
  void     (*read_work)       (RubricaCard* card, xmlNodePtr node);
  void     (*read_addresses)  (RubricaCard* card, xmlNodePtr node);
  void     (*read_net)        (RubricaCard* card, xmlNodePtr node);
  void     (*read_telephone)  (RubricaCard* card, xmlNodePtr node);
  void     (*read_notes)      (RubricaCard* card, xmlNodePtr node);

  gpointer      (*copy)            (RubricaCard* card); 

  void          (*display)         (RubricaCard* card, gpointer gui);
  RubricaInfos* (*get_infos)       (RubricaCard* card); 

  void          (*print_card)      (RubricaCard* card); 

  /* signals */
};



GType        rubrica_card_get_type (void);

RubricaCard* rubrica_card_new      (void);
void         rubrica_card_free     (RubricaCard* card);
RubricaCard* rubrica_card_copy     (RubricaCard* card);
void         rubrica_card_id_align (RubricaCard* card, 
				    gchar* id, gchar* card_type);

/*  Set
*/
void rubrica_card_set_id                (RubricaCard* card, gchar* id);
void rubrica_card_set_creation_time     (RubricaCard* card, time_t time);
void rubrica_card_set_modification_time (RubricaCard* card, time_t time);


/*  Get
*/
gchar* rubrica_card_get_id                (RubricaCard* card); 
time_t rubrica_card_get_creation_time     (RubricaCard* card);
time_t rubrica_card_get_modification_time (RubricaCard* card);

RubricaInfos* rubrica_card_get_infos      (RubricaCard* card);
void          rubrica_card_free_infos     (RubricaInfos *infos);

void rubrica_card_add_group       (RubricaCard* card, RubricaGroup* group);
void rubrica_card_add_address     (RubricaCard* card, RubricaAddress* address);
void rubrica_card_add_net_address (RubricaCard* card, RubricaNetAddress* net);
void rubrica_card_add_telephone   (RubricaCard* card, RubricaTelephone* tel);


gpointer rubrica_card_find_group      (RubricaCard* card, gchar* name);
gboolean rubrica_card_belong_to_group (RubricaCard* card, gchar* group_name);
gboolean rubrica_card_delete_group    (RubricaCard* card, gchar* name);
gboolean rubrica_card_rename_group    (RubricaCard* card, gchar* oldname, 
				       gchar* newname);


gboolean rubrica_card_delete_address  (RubricaCard* card,
				       RubricaAddress* address);
gboolean rubrica_card_replace_address (RubricaCard* card,
				       RubricaAddress* old, 
				       RubricaAddress* new);
/*
  TODO
  rubrica_card_delete_net_address
  rubrica_card_replace_net_address

  rubrica_card_delete_telephone
  rubrica_card_replace_telephone
*/

RubricaGroup* rubrica_card_get_group       (RubricaCard* card);
RubricaGroup* rubrica_card_get_next_group  (RubricaCard* card);
RubricaGroup* rubrica_card_get_prev_group  (RubricaCard* card);
void          rubrica_card_reset_group     (RubricaCard* card);

gpointer rubrica_card_get_address          (RubricaCard* card);
gpointer rubrica_card_get_next_address     (RubricaCard* card);
gpointer rubrica_card_get_prev_address     (RubricaCard* card);
void     rubrica_card_reset_address        (RubricaCard* card);

gpointer rubrica_card_get_net_address      (RubricaCard* card);
gpointer rubrica_card_get_next_net_address (RubricaCard* card);
gpointer rubrica_card_get_prev_net_address (RubricaCard* card);
void     rubrica_card_reset_net_address    (RubricaCard* card);

gpointer rubrica_card_get_telephone        (RubricaCard* card);
gpointer rubrica_card_get_next_telephone   (RubricaCard* card);
gpointer rubrica_card_get_prev_telephone   (RubricaCard* card);
void     rubrica_card_reset_telephone      (RubricaCard* card);

gchar*   rubrica_card_get_home_page        (RubricaCard* card);
gchar*   rubrica_card_get_email            (RubricaCard* card);
gchar*   rubrica_card_get_irc              (RubricaCard* card);
gchar*   rubrica_card_get_group_owner      (RubricaCard* card, 
					    RubricaGroup* group);


/* just for tests
 */
void rubrica_card_print     (RubricaCard* card);

#endif
