/*
*  RAL -- Rubrica Addressbook Library
*  file: groups.h
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_GROUP_SET_H
#define _RUBRICA_GROUP_SET_H

#include <glib.h>
#include <glib-object.h>

#include "group.h"

#define RUBRICA_GROUP_SET_TYPE           (rubrica_group_set_get_type())

#define RUBRICA_GROUP_SET(obj)           (G_TYPE_CHECK_INSTANCE_CAST((obj),  \
                                          RUBRICA_GROUP_SET_TYPE,            \
                                          RubricaGroupSet))

#define RUBRICA_GROUP_SET_CLASS(klass)   (G_TYPE_CHECK_CLASS_CAST((klass),   \
                                          RUBRICA_GROUP_SET_TYPE,            \
                                          RubricaGroupSetClass))

#define IS_RUBRICA_GROUP_SET(obj)        (G_TYPE_CHECK_INSTANCE_TYPE((obj),  \
                                          RUBRICA_GROUP_SET_TYPE))

#define IS_RUBRICA_GROUP_SET_CLASS(kls)  (G_TYPE_CHECK_CLASS_TYPE((kls),     \
                                          RUBRICA_GROUP_SET_TYPE))

#define RUBRICA_GROUP_SET_GET_CLASS(kls) (G_TYPE_INSTANCE_GET_CLASS((kls),   \
				          RUBRICA_GROUP_SET_TYPE,            \
                                          RubricaGroupSetClass))



typedef struct _RubricaGroupSet        RubricaGroupSet;
typedef struct _RubricaGroupSetClass   RubricaGroupSetClass;
typedef struct _RubricaGroupSetPrivate RubricaGroupSetPrivate;


struct _RubricaGroupSet {
  GObject parent;
  
  RubricaGroupSetPrivate* private;  
};


struct _RubricaGroupSetClass {
  GObjectClass parent;

  void (*group_added)   (RubricaGroupSet*, gpointer group);
  void (*group_removed) (RubricaGroupSet*, gpointer group);
};



GType            rubrica_group_set_get_type (void);



RubricaGroupSet* rubrica_group_set_new          (void);
void             rubrica_group_set_free         (RubricaGroupSet* set);


gpointer rubrica_group_set_find                 (RubricaGroupSet* set, 
						 gchar* group_name);
gboolean rubrica_group_set_owns_group           (RubricaGroupSet* set, 
						 gchar* group_name);

gboolean rubrica_group_set_add_group            (RubricaGroupSet* set,
						 RubricaGroup* group);
gboolean rubrica_group_set_delete_group         (RubricaGroupSet* set, 
						 RubricaGroup* group);
gboolean rubrica_group_set_delete_group_by_name (RubricaGroupSet* set,
						 gchar* name);
gboolean rubrica_group_set_rename_group         (RubricaGroupSet* set, 
						 gchar* oldname,
						 gchar* newname);
void     rubrica_group_set_disable_all          (RubricaGroupSet* set);
void     rubrica_group_set_enable_group         (RubricaGroupSet* set,
						 RubricaGroup* group);
void     rubrica_group_set_disable_group        (RubricaGroupSet* set,
						 RubricaGroup* group);


void          rubrica_group_set_reset           (RubricaGroupSet* set);
RubricaGroup* rubrica_group_set_get_group       (RubricaGroupSet* set);
RubricaGroup* rubrica_group_set_get_next_group  (RubricaGroupSet* set);
RubricaGroup* rubrica_group_set_get_prev_group  (RubricaGroupSet* set);

RubricaGroupSet* rubrica_group_set_merge_sets   (RubricaGroupSet* set,
						 RubricaGroupSet* second);
#endif
