/*
*  RAL -- Rubrica Addressbook Library
*  file: load.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <time.h>
#include <libintl.h>
#include <glib.h>
#include <libxml/tree.h>

#include "load.h"
#include "card.h"
#include "personal.h"
#include "company.h"
#include "contact.h"
#include "address.h"
#include "group.h"
#include "net.h"
#include "notes.h"
#include "telephone.h"
#include "io.h"
#include "error.h"

#define _(x) gettext(x)

/*
  RubricaDoc file structure:
     version:    rubrica's version that made the addressbook file
     fileformat: #4
     doctype:    addressbook

  <?xml version="1.0" encoding="UTF-8"?>
  <Rubrica version="" fileformat="" doctype="">        root
     <Card                                             root child 
            name=""                                    the card name
            deletable=""                               is a deletable card?
            deleted=""                                 was card deleted?
            rank=""                                    card rank (1..5)
            id="pcxxx"                                 card id   (es:  pc643)
	    created=""                                 creation date
            last_change=""                             last change date
	    type="personal">                           card's type 
         <Groups>                                      card child 
             <Group owner="rubrica" pixmap="">...</Group>   groups children
	     <Group owner="user" pixmap="">......</Group>
	     ...
         </Groups>       
	 <Addresses>                                   card child
             <Address type="">                         addresses children
                 <Street number="">....</Street>       address child
	         <City zip="">.........</City>
	         <Province>............</Province>
	         <State>...............</State>
	         <Country>.............</Country>
 	      </Address>
	     ...
         </Addresses>
	 <Net>
	     <Uri type="web">..........</Uri>
	     <Uri type="email">........</Uri>
	     <Uri type="aim">..........</Uri>
	     <Uri type="msn">..........</Uri>
	     <Uri type="yahoo">........</Uri>
	     <Uri type="icq">..........</Uri>
	     <Uri type="jabber">.......</Uri>	     
	     ...
	 </Net>
	 <TelephoneNumbers>
	     <Telephone type="home">...</Telephone>
	     <Telephone type="work">...</Telephone>
	     <Telephone type="fax">....</Telephone>
	     ...
 	 </TelephoneNumbers>
         <Data>                                        card child
             <FirstName>...............</FirstName>    data child 
	     <MiddleName>..............</MiddleName>     "    "
	     <LastName>................</LastName>                    
	     <Nickname>................</NickName>                    
	     <Profession>..............</Profession>                  
 	     <NamePrefix>..............</NamePrefix>                  
	     <Title>...................</Title>                       
	     <Photo type="">...........</Photo>
	     <Birthday known="" day="" month="" year=""/>  
         </Data>
	 <Work>
	     <Assignment>..............</Assignment>
	     <Organization>............</Organization>
             <Department>..............</Department>
	     <SubDepartment>...........</SubDepartment>
	     <ManagerName>.............</ManagerName>
	     <ManagerPhone>............</ManagerPhone>
	     <CollabratorName>.........</CollabratorName>
	     <CollaboratorPhone>.......</CollabratorPhone>
         </Work>
	 <Notes married="">
             <SpouseName>..............</SpouseName>
	     <Child>...................</Child>
	     <OtherNotes>..............</OtherNotes>
	     <PublicKey>...............</PublicKey>
	     <SpouseBirthday known="" day="" month="" year=""/>
	     <Anniversary known="" day="" month="" year""/>
         </Notes>
     </Card>

     <Card                                             root child 
            name=""                                    the card name
            deletable=""                               is a deletable card?
            deleted=""                                 was card deleted?
            rank=""                                    card rank (1..5)
            id="ccxxx"                                 card id   (es: cc1039)
	    created=""                                 creation date
            last_change=""                             last change date
	    type="company">                            card's type 
         <Groups>                                      card child 
             <Group>...................</Group>        groups children
	     ...
         </Groups>       
         <Company>
             <CompanyName>.............</CompanyName>
	     <Notes>...................</Notes>
         </Company>
	 <Addresses>                                   card child
             <Address type="">                         addresses children
                 <Street number="">....</Street>       address child
	         <City zip="">.........</City>
	         <Province>............</Province>
	         <State>...............</State>
	         <Country>.............</Country>
 	      </Address>
	     ...
         </Addresses>
	 <Net>
	     <Uri type="web">..........</Uri>
	     <Uri type="email">........</Uri> 
	     ...
	 </Net>
	 <TelephoneNumbers>
	     <Telephone type="green">......</Telephone>
	     <Telephone type="operator">...</Telephone>
	     <Telephone type="fax">........</Telephone>
	     ...
 	 </TelephoneNumbers>
	 <CompanyContacts>
	     <Contact ref_id="id_personal">first name, last name</Contact>
	     ...
	 <CompanyContacts>
     </Card>
     ...
  </Rubrica>


  Rubrica
     |
     | children
     |
     v   next         next         next 
   Card-------->Card-------->....------->Card
     |
     | children
     |
     v   next                
   Groups--->Data--->Addresses--->Net--->....Phone-->Work-->Notes
     |        \          |         |           |        |      |      
     | child  | child    | child   |           |        |      |     
     |        |          v         v           v        v      v     
     |        |         ...       ...         ...      ...     ...    ...   
     v        v
   Group    FirstName--->MiddleName--->...

   pcxxx => personal card #xxx
   ccxxx => company  card #xxx
*/



void   rubrica_load_real_load_card (RubricaCard* card, xmlNodePtr xmlcard);
gchar* rubrica_load_get_card_name  (xmlNodePtr node, RubricaError *err);


static RubricaGroupSet* set = NULL;


RubricaGroupSet*     
rubrica_load_get_user_group(void)
{
  return set;
}


gchar* 
rubrica_load_get_card_name(xmlNodePtr node, RubricaError *err)
{
  gchar* ret = NULL;

  ret = rubrica_io_get_prop(node, "name", err);

  if (!ret)
    {
//      rubrica_log_write(err);

      return NULL;
    }

  return (gchar*) ret;
}



void 
rubrica_load_read_infos(RubricaCard *card, xmlNodePtr node)
{
  xmlNodePtr xmlcard = NULL;
  RubricaError err;
  gchar* id;
  time_t t;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  xmlcard = rubrica_io_get_node(node, (xmlChar*) "Card");

  if (xmlcard)
    {
      gchar *name, *tmp;
      gboolean deletable,deleted; // , marked;
      RubricaRank rank;
 
      name      = rubrica_load_get_card_name(xmlcard, &err);
      deletable = rubrica_io_get_bool(xmlcard, "deletable", &err);
      deleted   = rubrica_io_get_bool(xmlcard, "deleted", &err);
      tmp       = rubrica_io_get_prop(xmlcard, "rank", &err); 

      if (tmp)
	{
	  rank = (RubricaRank) atoi(tmp);
	  g_free(tmp);
	}
      else 
	rank = RUBRICA_RANK_NORMAL;
      
      g_object_set(card, "card-name", name, "deletable", deletable, 
		   "deleted", deleted, "rank", rank, NULL);
      
      id = rubrica_io_get_prop(xmlcard, "id", &err);
      if (g_ascii_strcasecmp(id, "0") != 0)
	{
	  gchar* type = NULL;

	  type = rubrica_io_get_prop(xmlcard, "type", &err);
	  
	  rubrica_card_set_id (card, id);
	  rubrica_card_id_align(card, id, type); 
	}
      g_free(id);
      
      t = rubrica_io_get_date(xmlcard, "created", &err);
      rubrica_card_set_creation_time(card, t);
    
      t = rubrica_io_get_date(xmlcard, "last_change", &err);
      rubrica_card_set_modification_time(card, t);
    }
  else
    {
      /* TODO 
	 
      segnalare l'errore, far scegliere all'utente se 
      continuare a caricare automaticamente il file 
      o avviare un caricamento assistito
      */	 
    }
}



void 
rubrica_load_read_groups(RubricaCard *card, xmlNodePtr cardnode)
{
  xmlNodePtr xmltablegroups = NULL;
  xmlNodePtr xmlgroup;
  RubricaError err;

  /*  
      <Groups>                                          card child 
          <Group owner="" pixmap="">........</Group>    groups children
          ...
      </Groups>       
  */

  g_return_if_fail(IS_RUBRICA_CARD(card));

  xmltablegroups = rubrica_io_get_node(cardnode, (xmlChar*) "Groups");
  
  if (xmltablegroups)
    {
      xmlgroup = xmltablegroups->children;
      if (xmlIsBlankNode(xmlgroup))
	xmlgroup = xmlgroup->next;
      
      while (xmlgroup)
	{	  
	  RubricaGroup* group;
	  RubricaGroup* copy;
	  gchar* name;
	  gchar* owner;
	  gchar* pixmap;

	  if (xmlIsBlankNode(xmlgroup))
	    xmlgroup = xmlgroup->next;		  
	  
	  group = rubrica_group_new();
	  
	  name   = rubrica_io_get_content(xmlgroup, &err);
	  owner  = rubrica_io_get_prop(xmlgroup, "owner", &err);
	  pixmap = rubrica_io_get_prop(xmlgroup, "pixmap", &err);
		  
	  g_object_set(group, "group-name", name, "group-owner", owner,  
		       "group-pixmap", pixmap, "enabled", TRUE, NULL); 
		       
	  rubrica_card_add_group(card, group);
	  
	  if (g_ascii_strcasecmp(owner, "user") == 0)
	    {
	      if (!set)
		set = rubrica_group_set_new();

	      copy = rubrica_group_copy(group);
	      rubrica_group_set_add_group(set, copy);
	    }
	  
	  if (name)   g_free(name);
	  if (owner)  g_free(owner);
	  if (pixmap) g_free(pixmap);

	  xmlgroup = xmlgroup->next;
	  if (xmlIsBlankNode(xmlgroup))
	    xmlgroup = xmlgroup->next;
	} 
    }
}



void 
rubrica_load_read_contact(RubricaPersonalCard *card, xmlNodePtr cardnode)
{
  RubricaError err;
  RubricaContact* contact;
  xmlNodePtr xmldata = NULL;

  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(card));

  contact = rubrica_contact_new();
  if (!IS_RUBRICA_CONTACT(contact))
    {
      rubrica_personal_card_set_contact(card, NULL);
      return;
    }

  xmldata = rubrica_io_get_node(cardnode, (xmlChar*) "Data");
  
  if (xmldata)  
    {   
      gchar *day, *month, *year;
      gchar *first, *middle, *last, *nick, *prefix, *prof, *title, *photo;

      
      first  = rubrica_io_get(xmldata, "FirstName", &err);
      middle = rubrica_io_get(xmldata, "MiddleName", &err);
      last   = rubrica_io_get(xmldata, "LastName", &err);
      nick   = rubrica_io_get(xmldata, "NickName", &err);
      prof   = rubrica_io_get(xmldata, "Profession", &err);
      prefix = rubrica_io_get(xmldata, "NamePrefix", &err);
      title  = rubrica_io_get(xmldata, "Title", &err);
      photo  = rubrica_io_get(xmldata, "Photo", &err);

      g_object_set(contact, 
		   "first-name", first, "middle-name", middle, 
		   "last-name", last, "nick-name", nick, 
		   "prefix", prefix, "profession", prof, 
		   "title", title, "photo", photo, NULL);

      g_free(first);
      g_free(middle);
      g_free(last);
      g_free(nick);
      g_free(prefix);
      g_free(prof);
      g_free(title);
      g_free(photo);

      rubrica_io_get_calendar_from (xmldata, "Birthday", 
				    &day, &month, &year, &err);
      rubrica_contact_set_birthday (contact, atoi(day), atoi(month),
				    atoi(year));

      rubrica_personal_card_set_contact(card, contact);
    }
  else 
    g_warning((_("May be this file is corrupted. Can't read %s field"), 
	       "Data"));  
}


void 
rubrica_load_read_work(RubricaPersonalCard *card, xmlNodePtr cardnode)
{
  RubricaError err;
  xmlNodePtr xmlwork = NULL;

  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(card));

  xmlwork = rubrica_io_get_node(cardnode, (xmlChar*) "Work");

  if (xmlwork)
    {     
      RubricaWork* work;
      gchar *assignment, *org, *dep, *subdep;
      gchar *manager, *mphone, *collaborator, *cphone;

      work = rubrica_work_new();
      
      if (!IS_RUBRICA_WORK(work))
	return;
      
      assignment   = rubrica_io_get(xmlwork, "Assignment", &err);      
      org          = rubrica_io_get(xmlwork, "Organization", &err);
      dep          = rubrica_io_get(xmlwork, "Department", &err);
      subdep       = rubrica_io_get(xmlwork, "SubDepartment", &err);
      manager      = rubrica_io_get(xmlwork, "ManagerName", &err);
      mphone       = rubrica_io_get(xmlwork, "ManagerPhone", &err);
      collaborator = rubrica_io_get(xmlwork, "CollaboratorName", &err);
      cphone       = rubrica_io_get(xmlwork, "CollaboratorPhone", &err);

      g_object_set(work, 
		   "assignment", assignment, "organization", org, 
		   "department", dep, "sub-department", subdep, 
		   "manager-name", manager, "manager-phone", mphone, 
		   "collaborator", collaborator, "collaborator-phone", cphone,
		   NULL);

      g_free(assignment);
      g_free(org);
      g_free(dep);
      g_free(subdep);
      g_free(manager);
      g_free(mphone);
      g_free(collaborator);
      g_free(cphone);

      rubrica_personal_card_set_work(card, work);
    }
  else
    {
      g_warning((_("May be this file is corrupted. Can't read %s field"), 
		 "Work"));
    }
}



void  
rubrica_load_read_addresses(RubricaCard *card, xmlNodePtr cardnode)
{
  RubricaAddress* address = NULL;
  RubricaError err;
  xmlNodePtr xmladd = NULL;
  xmlNodePtr xmladdress;
  gchar *tmp;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  xmladd = rubrica_io_get_node(cardnode, (xmlChar*) "Addresses");

  if (xmladd)
    {
      RubricaAddressType type;
      gchar *street, *number, *city, *zip;
      gchar *province, *state, *country;

      /* child is the head of an <address> list */
      xmladdress = xmladd->children;  
      if (xmlIsBlankNode(xmladdress))
	xmladdress = xmladdress->next;
      
      while(xmladdress)
	{
	  address = rubrica_address_new();
	  
	  if (!IS_RUBRICA_ADDRESS(address))
	    {
	      g_warning("address obj get wrong type");

	      return;
	    }
	  
	  
	  /* The address's data:

	    <Address type="">                         addresses children
                <Street number="">....</Street>       address child 
		...
	    </Address>
	  */

	  /* get address type --- <Address type=""> 
	   */
	  tmp      = rubrica_io_get_prop(xmladdress, "type", &err);
	  type     = rubrica_address_encode_type(tmp);
	  street   = rubrica_io_get(xmladdress, "Street", &err);
	  number   = rubrica_io_get_prop_from(xmladdress, "Street", "number",
					      &err);
	  city     = rubrica_io_get(xmladdress, "City", &err);
	  zip      = rubrica_io_get_prop_from(xmladdress, "City", "zip", &err);
	  province = rubrica_io_get(xmladdress, "Province", &err);
	  state    = rubrica_io_get(xmladdress, "State", &err);
	  country  = rubrica_io_get(xmladdress, "Country", &err);
	  
	  g_object_set(address, "address-type", type, "street", street,  
		       "street-number", number, "city", city, "zip", zip,  
		       "province", province, "state", state,  
		       "country", country, NULL); 
	  
	  rubrica_card_add_address(card, address);

	  xmladdress = xmladdress->next;
	  if (xmlIsBlankNode(xmladdress)) 
	    xmladdress = xmladdress->next;
	}
    }
  else 
    {
      g_warning((_("May be this file is corrupted. Can't read %s field"), 
		 "Address"));
    }
}


void
rubrica_load_read_net(RubricaCard *card, xmlNodePtr cardnode)
{
  RubricaNetAddress* net;
  RubricaError err;
  xmlNodePtr xmlnet_child;
  xmlNodePtr xmlnet = NULL;
  gchar* net_data;
  gchar* net_type;

  /*
    <Net>
       <Uri type="web">..........</Uri>
       <Uri type="email">........</Uri> 
       <Uri type="icq">..........</Uri>
       ...
    </Net>
  */
  g_return_if_fail(IS_RUBRICA_CARD(card));

  xmlnet = rubrica_io_get_node(cardnode, (xmlChar*) "Net");

  if (xmlnet)
    {
      xmlnet_child = xmlnet->children;
      if (xmlIsBlankNode(xmlnet_child))
	xmlnet_child = xmlnet_child->next;

      while (xmlnet_child)
	{
	  RubricaNetAddressType type;

	  if (xmlIsBlankNode(xmlnet_child))
	    xmlnet_child = xmlnet_child->next;
	  
	  net_data = rubrica_io_get_content(xmlnet_child, &err);
	  net_type = rubrica_io_get_prop(xmlnet_child, "type", &err);

	  if (net_data)
	    {
	      net = rubrica_net_address_new();
	      
	      if(!IS_RUBRICA_NET_ADDRESS(net))
		{
		  g_warning("net obj get wrong type");
		  
		  g_free(net_type);
		  g_free(net_data);

		  return;
		}
	      
	      type = rubrica_net_address_encode_type(net_type);
	      g_object_set(net, "url", net_data, "url-type", type, NULL);
	      rubrica_card_add_net_address(card, net);
	      
	      g_free(net_data);
	      g_free(net_type);
	    }	  

	  xmlnet_child = xmlnet_child->next;
	  if (xmlIsBlankNode(xmlnet_child)) 
	    xmlnet_child = xmlnet_child->next;
	}
    }
  else
    {
      // errore
    }
}


/* 

void 
rubrica_io_load_web_data(RubricaPersonalCard *card, xmlNodePtr node)
{
  RubricaError err;
  xmlNodePtr child;
  RubricaCardNetUri* uri;
  xmlNodePtr xmlweb = NULL;
  gchar *tmp;
  
  xmlweb = rubrica_io_get_node(node, "WebAddresses");

  if (xmlweb)
    {
      child = xmlweb->children;
      if (xmlIsBlankNode(child))
	child = child->next;

      while (child)
	{
	  if (xmlIsBlankNode(child))
	    child = child->next;

	  tmp = rubrica_io_get_content(child, &err);
	  if (tmp)
	    {
	      uri = g_malloc (sizeof(RubricaCardNetUri));
	      
	      if (!uri)
		{
		  g_warning("Out of memory");
		  if (personal->net)
		    personal->net = g_list_append(personal->net, NULL);
		  
		  return;	      
		}

	      uri->url  = tmp;
	      uri->type = g_strdup("web");
	      
	      personal->net = g_list_append(personal->net, uri);
	    }
	  
	  child = child->next;
	}
    }
  else
    {
      // errore
    }
}



void 
rubrica_io_load_email_data(RubricaPersonalCard *card, xmlNodePtr node)
{
  RubricaError err;
  RubricaCardNetUri* uri;
  xmlNodePtr xmlemail = NULL;
  xmlNodePtr child;
  gchar *tmp;
  
  xmlemail = rubrica_io_get_node(node, "EmailAddresses");
  
  if (xmlemail)
    {
      child = xmlemail->children;
      if (xmlIsBlankNode(child))
	child = child->next;

      while (child)
	{
	  if (xmlIsBlankNode(child))
	    child = child->next;

	  tmp = rubrica_io_get_content(child, &err);
	  if (tmp)
	    {
	      uri = g_malloc (sizeof(RubricaCardNetUri));
	      
	      if (!uri)
		{
		  g_warning("Out of memory");
		  if (personal->net)
		    personal->net = g_list_append(personal->net, NULL);
		  
		  return;	      
		}

	      uri->url  = tmp;
	      uri->type = g_strdup("email");
	      
	      personal->net = g_list_append(personal->net, uri);
	    }
	  
	  child = child->next;
	}
    }
  else
    {
      // errore
    }  

}

*/


void
rubrica_load_read_telephone(RubricaCard *card, xmlNodePtr cardnode)
{
  xmlNodePtr xmltel_child;
  xmlNodePtr xmltel = NULL;
  RubricaError err;
  RubricaTelephone* tel;
  RubricaTelephoneType ttype;
  gchar* number;
  gchar* type;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  xmltel = rubrica_io_get_node(cardnode, (xmlChar*) "TelephoneNumbers");

  if (xmltel)
    {
      xmltel_child = xmltel->children;
      if (xmlIsBlankNode(xmltel_child))
	xmltel_child = xmltel_child->next;

      while(xmltel_child)
	{
	  if (xmlIsBlankNode(xmltel_child))
	    xmltel_child = xmltel_child->next;
	  
	  number = rubrica_io_get_content(xmltel_child, &err);
	  type   = rubrica_io_get_prop(xmltel_child, "type", &err);

	  if (number)
	    {
	      tel = rubrica_telephone_new();
	      
	      if(!IS_RUBRICA_TELEPHONE(tel))
		{
		  g_warning("telephone obj get wrong type");
		  
		  return;
		}
	      
	      ttype = rubrica_telephone_encode_type(type);
	      g_object_set(tel, "telephone-number", number, 
			   "telephone-type", ttype, NULL);

	      rubrica_card_add_telephone(card, tel);

	      g_free(number);
	      g_free(type);
	    }
	  
	  xmltel_child = xmltel_child->next;
	  if (xmlIsBlankNode(xmltel_child)) 
	    xmltel_child = xmltel_child->next;
	} 
    }
  else
    {
      g_warning((_("May be this file is corrupted. Can't read %s field"), 
		 "TelephoneNumbers"));
    }
}


void 
rubrica_load_read_notes(RubricaPersonalCard *card, xmlNodePtr cardnode)
{
  RubricaError err;
  xmlNodePtr xmlnote = NULL;

  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(card));

  xmlnote = rubrica_io_get_node(cardnode, (xmlChar*) "Notes");
  
  if (xmlnote)
    {      
      RubricaNotes* notes;
      gchar *day, *month, *year;
      gchar *partner, *children, *other, *pubkey;
      gboolean bool, has_partner;

      notes = rubrica_notes_new();

      if (!IS_RUBRICA_NOTES(notes))
	return;

      has_partner = rubrica_io_get_bool(xmlnote, "partner", &err);
      partner     = rubrica_io_get(xmlnote, "PartnerName", &err);
      children    = rubrica_io_get(xmlnote, "Child", &err);
      other       = rubrica_io_get(xmlnote, "OtherNotes", &err);
      pubkey      = rubrica_io_get(xmlnote, "PublicKey", &err);

      g_object_set(notes, "has-partner", has_partner, "partner-name", partner,
		   "children", children, "other-notes", other, 
		   "pubkey", pubkey, NULL);

      g_free(partner);
      g_free(children);
      g_free(other);
      g_free(pubkey);
  
      bool = rubrica_io_get_bool_from(xmlnote, "PartnerBirthday", 
				      "known", &err);
      rubrica_io_get_calendar_from (xmlnote, "PartnerBirthday", 
				    &day, &month, &year, &err);
      if (bool)
	{
	  rubrica_notes_set_know_birthday(notes, TRUE);
	  rubrica_notes_set_birthday (notes, atoi(day), 
				      atoi(month), atoi(year));
	}

      bool = rubrica_io_get_bool_from(xmlnote, "Anniversary", "known", &err);
      rubrica_io_get_calendar_from (xmlnote, "Anniversary",&day, &month,
				    &year, &err);
      if (bool)
	{
	  rubrica_notes_set_know_anniversary(notes, TRUE);
	  rubrica_notes_set_anniversary(notes, atoi(day), 
					atoi(month), atoi(year)); 
	}

      rubrica_personal_card_set_notes (card, notes);
    } 
  else
    {
      g_warning((_("May be this file is corrupted. Can't read %s field"), 
		 "Notes"));
    }
}



void 
rubrica_load_read_company (RubricaCompanyCard* card, xmlNodePtr cardnode)
{
  RubricaError err;
  xmlNodePtr xmlcompany = NULL;
  gchar* tmp;

  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(card));

  xmlcompany = rubrica_io_get_node(cardnode, (xmlChar*) "Company");

  if (xmlcompany)
    {      
      tmp = rubrica_io_get(xmlcompany, "CompanyName", &err);
      g_object_set(card, "company-name", tmp, NULL);
      g_free(tmp);

      tmp = rubrica_io_get(xmlcompany, "PI", &err);
      g_object_set(card, "pi", tmp, NULL);
      g_free(tmp);

      tmp = rubrica_io_get(xmlcompany, "Notes", &err);
      g_object_set(card, "notes", tmp, NULL);
      g_free(tmp);
    }
  else   
    {
      g_warning((_("May be this file is corrupted. Can't read %s field"), 
		 "Company"));
    }
}





/*    Public
*/

/*  open the file "filename"
    return ptr to xml doc or NULL on error
 */
xmlDocPtr 
rubrica_load_open_doc(gchar *filename, RubricaError* err)
{
  xmlDocPtr doc;

  *err = NO_FILENAME;
  g_return_val_if_fail(filename != NULL, NULL);
  
  /* read and parse file 
   */
  doc = xmlParseFile(filename);

  if (!doc)
    {
      *err = PARSING_DOC;
      
      return NULL;
    }  

  *err = NO_ERROR;
  return doc;
}


/*  get the file format of the rubrica's doc    
*/
gint 
rubrica_load_get_fileformat(xmlDocPtr doc, RubricaError* err)
{
  gchar* tmp;
  gint ret;

  *err = NO_RUBRICA_DOC;
  g_return_val_if_fail(doc != NULL, -1);

  if(!xmlHasProp(doc->children, (xmlChar*) "fileformat"))
    {
      *err = UNKNOWN_FILEFORMAT;
      return 0;    
    }

  *err = NO_ERROR;
  tmp = (gchar*) xmlGetProp(doc->children, (xmlChar*) "fileformat");
  ret = atoi(tmp);
  xmlFree(tmp);

  return ret;
}


/*  get version.
    the version of rubrica that make the file
*/
gchar*
rubrica_load_get_version(xmlDocPtr doc, RubricaError *err)
{
  gchar* ret = NULL;
  *err = NO_RUBRICA_DOC;
  g_return_val_if_fail(doc != NULL, NULL);
  
  if(!xmlHasProp(doc->children, (xmlChar*) "version"))
    {
      *err = UNKNOWN_VERSION;
      return NULL;
    }

  *err = NO_ERROR;
  ret = (gchar*) xmlGetProp(doc->children, (xmlChar*) "version");

  return ret;
}


/*  get the document type
    values: ADDRESSBOOK_TYPE_DOC
            ACCOUNT_TYPE_DOC         // removed, may be added in future
*/
gchar*
rubrica_load_get_document_tpye(xmlDocPtr doc, RubricaError *err)
{
  gchar* ret = NULL;

  *err = NO_RUBRICA_DOC;
  g_return_val_if_fail(doc != NULL, NULL);
  
  if(!xmlHasProp(doc->children, (xmlChar*) "doctype"))
    {
      *err = UNKNOWN_DOC;
      return NULL;
    }

  *err = NO_ERROR;
  ret  = (gchar*) xmlGetProp(doc->children, (xmlChar*) "doctype");

  return ret;
}


/*  check if file "filename" is a rubrica doc
    return TRUE if xml doc is a rubrica doc FALSE otherwise
*/
gboolean 
rubrica_load_is_rubrica_doc(xmlDocPtr doc, RubricaError *err)
{
  *err = NO_RUBRICA_DOC;
  g_return_val_if_fail(doc != NULL, FALSE);

  if (!doc->children || 
      !doc->children->name || 
      g_ascii_strcasecmp((gchar*) doc->children->name, 
			 (gchar*) "Rubrica") != 0)
    { 
      *err = UNKNOWN_DOC;
//      rubrica_dialog_error_loading_file(fname, RE_NO_RUBRICA_HEADER);  

      return FALSE;
    }  

  *err = NO_ERROR;

  return TRUE;
}


RubricaPersonalCard* 
rubrica_load_read_personal_xmlcard(xmlNodePtr xmlcard)
{
  RubricaPersonalCard* card;

  card = rubrica_personal_card_new();
  rubrica_load_real_load_card(RUBRICA_CARD(card), xmlcard);

  return card;
}

RubricaCompanyCard *
rubrica_load_read_company_xmlcard(xmlNodePtr xmlcard)
{
  RubricaCompanyCard *card;

  card = rubrica_company_card_new();
  rubrica_load_real_load_card(RUBRICA_CARD(card), xmlcard);

  return card;
}


void
rubrica_load_real_load_card(RubricaCard* card, xmlNodePtr xmlcard)
{
  RubricaCardClass* klass;

  klass = RUBRICA_CARD_GET_CLASS(card); 
  
  /*   Card data
   */
  if (RUBRICA_CARD_CLASS(klass)->read_infos)  
    RUBRICA_CARD_CLASS(klass)->read_infos(card, xmlcard);

  /*   Card groups
  */
  if (RUBRICA_CARD_CLASS(klass)->read_group)  
    RUBRICA_CARD_CLASS(klass)->read_group(card, xmlcard);
  
  /*    Personal (Contact or Company) data
   */
  if (RUBRICA_CARD_CLASS(klass)->read_contact)  
    RUBRICA_CARD_CLASS(klass)->read_contact(card, xmlcard);
  
  /*    Work data
  */
  if (RUBRICA_CARD_CLASS(klass)->read_work)  
    RUBRICA_CARD_CLASS(klass)->read_work(card, xmlcard);
  
  /*    Addresses
   */
  if (RUBRICA_CARD_CLASS(klass)->read_addresses)  
    RUBRICA_CARD_CLASS(klass)->read_addresses(card, xmlcard);

  /*    NetAddresses
   */
  if (RUBRICA_CARD_CLASS(klass)->read_net)  
    RUBRICA_CARD_CLASS(klass)->read_net(card, xmlcard);

  /*    Telephone numbers
  */
  if (RUBRICA_CARD_CLASS(klass)->read_telephone)  
    RUBRICA_CARD_CLASS(klass)->read_telephone(card, xmlcard);

  /*    Notes
   */
  if (RUBRICA_CARD_CLASS(klass)->read_notes)  
    RUBRICA_CARD_CLASS(klass)->read_notes(card, xmlcard);  
}


RubricaPersonalCard* 
rubrica_load_read_old_personal_xmlcard(xmlNodePtr xmlcard)
{
  RubricaPersonalCard* card = NULL;

 /*
  card = rubrica_personal_card_new();
  
  rubrica_load_load_card_data(RUBRICA_CARD(card), xmlcard); 
  rubrica_load_load_groups_data(RUBRICA_CARD(card), xmlcard);
  rubrica_load_load_contact_data(card, xmlcard);
  rubrica_load_load_address_data(RUBRICA_CARD(card), xmlcard);  
  rubrica_load_load_net_data(RUBRICA_CARD(card), xmlcard);  
  rubrica_load_load_phone_data(RUBRICA_CARD(card), xmlcard);
  rubrica_load_load_work_data(card, xmlcard);
  rubrica_load_load_note_data(card, xmlcard);
  */
  return card;
}
