/*
*  RAL -- Rubrica Addressbook Library
*  file: notes.h
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_NOTES_H
#define _RUBRICA_NOTES_H

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "date.h"

#define RUBRICA_NOTES_TYPE             (rubrica_notes_get_type())

#define RUBRICA_NOTES(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                        RUBRICA_NOTES_TYPE,               \
                                        RubricaNotes))

#define RUBRICA_NOTES_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass),  \
                                        RUBRICA_NOTES_TYPE,               \
                                        RubricaNotesClass))

#define IS_RUBRICA_NOTES(obj)          (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
                                        RUBRICA_NOTES_TYPE))

#define IS_RUBRICA_NOTES_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE((klass),  \
                                        RUBRICA_NOTES_TYPE))

#define RUBRICA_NOTES_GET_CLASS(klass) (G_TYPE_INSTANCE_GET_CLASS((kls),  \
                                        RUBRICA_NOTES_TYPE,               \
                                        RubricaNotesClass))



typedef struct _RubricaNotes        RubricaNotes; 
typedef struct _RubricaNotesClass   RubricaNotesClass;
typedef struct _RubricaNotesPrivate RubricaNotesPrivate;


struct _RubricaNotes
{
  GObject parent;
  
  RubricaNotesPrivate* private;
};


struct _RubricaNotesClass
{
  GObjectClass parent_class;
};


GType         rubrica_notes_get_type (void);
 

RubricaNotes* rubrica_notes_new  (void);
void          rubrica_notes_free (RubricaNotes* notes);

RubricaNotes* rubrica_notes_copy (RubricaNotes* notes);

/*    set
*/
void rubrica_notes_set_know_birthday    (RubricaNotes* notes, gboolean know);
void rubrica_notes_set_know_anniversary (RubricaNotes* notes, gboolean know);


void rubrica_notes_set_birthday         (RubricaNotes* notes, gint day,
					 gint month, gint year);
void rubrica_notes_set_anniversary      (RubricaNotes* notes, gint day,
					 gint month, gint year);


/*    get
 */
gboolean rubrica_notes_know_birthday         (RubricaNotes* notes);
gboolean rubrica_notes_know_anniversary      (RubricaNotes* notes);


gchar*   rubrica_notes_get_birth_day         (RubricaNotes* notes);
gchar*   rubrica_notes_get_birth_month       (RubricaNotes* notes);
gchar*   rubrica_notes_get_birth_year        (RubricaNotes* notes);

gchar*   rubrica_notes_get_anniversary_day   (RubricaNotes* notes);
gchar*   rubrica_notes_get_anniversary_month (RubricaNotes* notes);
gchar*   rubrica_notes_get_anniversary_year  (RubricaNotes* notes);

gchar*   rubrica_notes_get_birthdate         (RubricaNotes* notes);
gchar*   rubrica_notes_get_anniversarydate   (RubricaNotes* notes);


void     rubrica_notes_append_other_notes    (RubricaNotes* notes, 
					      gchar* other_note);
#endif
