/*
*  RAL -- Rubrica Addressbook Library
*  file: vcard.c
*
*  the vcard import engine v0.5 
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <glib.h>
#include <glib-object.h>

#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "vcard.h"
#include "card.h"
#include "personal.h"
#include "company.h"
#include "contact.h"
#include "work.h"
#include "notes.h"
#include "address.h"
#include "net.h"
#include "telephone.h"
#include "ref.h"
#include "error.h"

#define BUFLEN    128
#define ARRAY_LEN 8


#define TOKEN_DELIM ':'
#define PARAM_DELIM ';'
#define VALUE_DELIM '='
#undef  VERSION


typedef enum {
  N = 1, 
  FN, 
  TZ, 
  END, 
  ADR, 
  TEL, 
  GEO, 
  ORG, 
  REV, 
  UID, 
  URL, 
  KEY, 
  BDAY, 
  ROLE, 
  LOGO, 
  NOTE, 
  BEGIN,
  PHOTO, 
  LABEL,
  EMAIL, 
  TITLE, 
  AGENT, 
  PROID, 
  SOUND, 
  CLASS,
  MAILER, 
  VERSION,
  NICKNAME, 
  CATEGORIES,                  // last vcard token

  KDE_EXT_Profession,          // KDE extensions
  KDE_EXT_Department,
  KDE_EXT_ManagersName,
  KDE_EXT_AssistantsName,
  KDE_EXT_Office,
  KDE_EXT_SpousesName,
  KDE_EXT_Anniversary,
  KDE_EXT_CUSTOM,

  RUBRICA_EXT_Group,           // Rubrica extensions
  RUBRICA_EXT_CompanyName,
  RUBRICA_EXT_Street,
  RUBRICA_EXT_StreetNumber,
  RUBRICA_EXT_ZipCode,
  RUBRICA_EXT_City,
  RUBRICA_EXT_Province,
  RUBRICA_EXT_Country,
  RUBRICA_EXT_Web,
  RUBRICA_EXT_Email,
  RUBRICA_EXT_Operator,
  RUBRICA_EXT_Fax,
  RUBRICA_EXT_Green,
  RUBRICA_EXT_CustomerCare,
  RUBRICA_EXT_Notes,
  RUBRICA_EXT_Department,
  RUBRICA_EXT_SubDepartment,
  RUBRICA_EXT_ManagerName,
  RUBRICA_EXT_CollaboratorName,
  RUBRICA_EXT_CollaboratorPhone,
  RUBRICA_EXT_SpouseName,
  RUBRICA_EXT_Child,
  RUBRICA_EXT_Anniversary,

  EVOLUTION_EXT_File_as,       // Evolution extensions
  EVOLUTION_EXT_Office,
  EVOLUTION_EXT_Manager,
  EVOLUTION_EXT_Assistant,
  EVOLUTION_EXT_Spouse,
  EVOLUTION_EXT_Anniversary,

  MOZILLA_EXT_Html,

  BAD_TOKEN,                   // this isn't a VCard token
} RubricaVCardTokenType;


typedef enum {
  TYPE = 1, 
  VALUE, 
  ENCODING,
  BAD_PARAM,                   // this isn't a VCard token
} RubricaVCardParamType;


typedef enum {
  VCARD = 1,
  BINARY,
  BASIC, 
  PUBLIC, 
  PRIVATE, 
  CONFIDENTIAL,
  HOME,  
  WORK, 
  POSTAL, 
  PARCEL, 
  DOM, 
  INTL, 
  PREF,
  VOICE, 
  FAX, 
  MSG, 
  CELL, 
  PAGER, 
  BBS, 
  MODEM, 
  CAR, 
  ISDN, 
  VIDEO,  
  AOL, 
  INTERNET,
  APPLELINK, 
  ATTMAIL, 
  CIS, 
  EWORLD, 
  IBMMail, 
  MCIMAIL, 
  POWERSHARE, 
  PRODIGY, 
  TLX, 
  X400, 
  PGP, 
  X509, 
  TEXT,
  URI,
  JPEG,
  QUOTED_PRINTABLE,

  BAD_VALUE,                 // this isn't a vcard token
} RubricaVCardValueType;



typedef struct {
  gchar *str;                /* token */
  gint tok;                  /* vcard decoded token */
} RubricaVCardType;


static RubricaVCardType tokens[] = { 
  {"N",   N},             {"FN",  FN},        {"TZ",  TZ},
  {"END", END},           {"ADR", ADR},       {"TEL", TEL},
  {"GEO", GEO},           {"ORG", ORG},       {"REV", REV},
  {"UID", UID},           {"URL", URL},       {"KEY", KEY},
  {"BDAY", BDAY},         {"ROLE", ROLE},     {"LOGO", LOGO}, 
  {"NOTE", NOTE},         {"BEGIN", BEGIN},   {"PHOTO", PHOTO},  
  {"LABEL", LABEL},       {"EMAIL", EMAIL},   {"TITLE", TITLE}, 
  {"AGENT", AGENT},       {"PROID", PROID},   {"SOUND", SOUND},  
  {"CLASS", CLASS},       {"MAILER", MAILER}, {"VERSION", VERSION}, 
  {"NICKNAME", NICKNAME}, {"CATEGORIES", CATEGORIES},

  /* kde extension.
  */
  {"X-KADDRESSBOOK-X-Profession",     KDE_EXT_Profession},
  {"X-KADDRESSBOOK-X-Department",     KDE_EXT_Department},
  {"X-KADDRESSBOOK-X-ManagersName",   KDE_EXT_ManagersName},
  {"X-KADDRESSBOOK-X-AssistantsName", KDE_EXT_AssistantsName},
  {"X-KADDRESSBOOK-X-Office",         KDE_EXT_Office},
  {"X-KADDRESSBOOK-X-SpousesName",    KDE_EXT_SpousesName},
  {"X-KADDRESSBOOK-X-Anniversary",    KDE_EXT_Anniversary},
  {"X-KADDRESSBOOK-X-CUSTOM",         KDE_EXT_CUSTOM},
  
  /* rubrica extension. 
     So rubrica can import a previously 
     exported rubrica's addressbook to vcard format
  */
  {"X-RUBRICA-X-Group",             RUBRICA_EXT_Group},
  {"X-RUBRICA-X-CompanyName",       RUBRICA_EXT_CompanyName},
  {"X-RUBRICA-X-Street",            RUBRICA_EXT_Street},
  {"X-RUBRICA-X-StreetNumber",      RUBRICA_EXT_StreetNumber},
  {"X-RUBRICA-X-ZipCode",           RUBRICA_EXT_ZipCode},
  {"X-RUBRICA-X-City",              RUBRICA_EXT_City},
  {"X-RUBRICA-X-Province",          RUBRICA_EXT_Province},
  {"X-RUBRICA-X-Country",           RUBRICA_EXT_Country},
  {"X-RUBRICA-X-Web",               RUBRICA_EXT_Web},
  {"X-RUBRICA-X-Email",             RUBRICA_EXT_Email},
  {"X-RUBRICA-X-Operator",          RUBRICA_EXT_Operator},
  {"X-RUBRICA-X-Fax",               RUBRICA_EXT_Fax},
  {"X-RUBRICA-X-Green",             RUBRICA_EXT_Green},
  {"X-RUBRICA-X-CustomerCare",      RUBRICA_EXT_CustomerCare},
  {"X-RUBRICA-X-Department",        RUBRICA_EXT_Department},
  {"X-RUBRICA-X-SubDepartment",     RUBRICA_EXT_SubDepartment},
  {"X-RUBRICA-X-ManagerName",       RUBRICA_EXT_ManagerName},
  {"X-RUBRICA-X-CollaboratorName",  RUBRICA_EXT_CollaboratorName},
  {"X-RUBRICA-X-CollaboratorPhone", RUBRICA_EXT_CollaboratorPhone},
  {"X-RUBRICA-X-SpouseName",        RUBRICA_EXT_SpouseName},
  {"X-RUBRICA-X-Child",             RUBRICA_EXT_Child},
  {"X-RUBRICA-X-Anniversary",       RUBRICA_EXT_Anniversary},

  /* evolution extensions
  */
  {"X-EVOLUTION-FILE-AS",        EVOLUTION_EXT_File_as},  
  {"X-EVOLUTION-OFFICE",         EVOLUTION_EXT_Office},
  {"X-EVOLUTION-MANAGER",        EVOLUTION_EXT_Manager},  
  {"X-EVOLUTION-ASSISTANT",      EVOLUTION_EXT_Assistant},
  {"X-EVOLUTION-SPOUSE",         EVOLUTION_EXT_Spouse},  
  {"X-EVOLUTION-ANNIVERSARY",    EVOLUTION_EXT_Anniversary},

  {"X-MOZILLA-HTML",             MOZILLA_EXT_Html},

  {NULL,  BAD_TOKEN}
};
	
	
static RubricaVCardType paramtokens[] = {
  {"type",     TYPE},
  {"value",    VALUE},   
  {"encoding", ENCODING},
  {NULL,       BAD_PARAM}
};		


static RubricaVCardType valuetokens[] = {
  {"home",     HOME},     {"work",       WORK},       {"cell",    CELL},     
  {"fax",      FAX},      {"postal",     POSTAL},     {"parcel",  PARCEL},  
  {"dom",      DOM},      {"intl",       INTL},       {"pref",    PREF},    
  {"voice",    VOICE},    {"msg",        MSG},        {"pager",   PAGER},   
  {"bbs",      BBS},      {"modem",      MODEM},      {"car",     CAR},     
  {"isdn",     ISDN},     {"video",      VIDEO},      {"aol",     AOL},     
  {"internet", INTERNET}, {"applelink",  APPLELINK},  {"attmail", ATTMAIL},
  {"cis",      CIS},      {"eworld",     EWORLD},     {"ibmmail", IBMMail},  
  {"mcimail",  MCIMAIL},  {"powershare", POWERSHARE}, {"prodigy", PRODIGY}, 
  {"tlx",      TLX},      {"x400",       X400},       {"vcard",   VCARD},
  {"pgp",      PGP},      {"x509",       X509},       {"b",       BINARY},   
  {"binary",   BINARY},   {"text",       TEXT},       {"uri",     URI},
  {"basic",    BASIC},    {"jpeg",       JPEG},       {"public",  PUBLIC},
  {"private",  PRIVATE},  {"confidential", CONFIDENTIAL}, 
  {"quoted-printable", QUOTED_PRINTABLE},  
  {NULL,       BAD_VALUE}
};


static gboolean have_company         = FALSE;
static gboolean have_company_address = FALSE;

gchar*   tmpbuf = NULL;
gchar*   decoded[8];
gboolean multiline_data = FALSE;

RubricaVCardTokenType oldtoken;
RubricaVCardParamType oldparam[ARRAY_LEN];
RubricaVCardValueType oldvalue[ARRAY_LEN];

RubricaPersonalCard   *card              = NULL;
RubricaCompanyCard    *company           = NULL;
RubricaContact        *contact           = NULL;
RubricaWork           *work              = NULL;
RubricaNotes          *notes             = NULL;
RubricaAddress        *address           = NULL;
RubricaTelephone      *tel               = NULL;
RubricaNetAddress     *net               = NULL;
RubricaAddress        *company_address   = NULL;
RubricaTelephone      *company_telephone = NULL;
RubricaNetAddress     *company_net       = NULL;


/*    signals enumeration 
 */
enum {
  VCARD_FILE_OPENED,              // vcard file was/wasn't opened
  VCARD_CARD_DECODED,             // vcard item was decoded
  VCARD_ADDRESSBOOK_DECODED,      // vcard file was decoded
  LAST_SIGNAL
};

static guint rubrica_vcard_signals[LAST_SIGNAL] = {0};


static void   rubrica_vcard_init        (RubricaVCard* obj);
static void   rubrica_vcard_class_init  (RubricaVCardClass* klass);

static void   rubrica_vcard_dispose     (RubricaVCard* obj);
static void   rubrica_vcard_finalize    (RubricaVCard* obj);


/* private functions
*/
static RubricaCompanyCard*   rubrica_vcard_company_new (RubricaCard* card); 
static RubricaAddress*       rubrica_vcard_company_address_new (void);

static gchar*                rubrica_vcard_append_data (gchar* buf, 
							gchar* data);
static gint                  rubrica_vcard_split_data  (gchar* data);

static RubricaVCardTokenType rubrica_vcard_decode_token(gchar* string);
static RubricaVCardValueType rubrica_vcard_decode_value(gchar* string);
static RubricaVCardParamType rubrica_vcard_decode_param(gchar* string);
static time_t                rubrica_vcard_decode_date (gchar* date, 
							gboolean has_hour);


static gchar* rubrica_vcard_extract_tokens (gchar* buffer,
					    RubricaVCardTokenType* token,
					    RubricaVCardParamType* param,
					    RubricaVCardValueType* value);

static void   rubrica_vcard_route_tokens   (RubricaVCard* vcard,
					    gchar* buffer, gchar* data, 
					    RubricaVCardTokenType token, 
					    RubricaVCardParamType* parray, 
					    RubricaVCardValueType* varray);

static void   rubrica_vcard_build_card     (RubricaVCard *vcard, 
					    RubricaVCardTokenType token,
					    RubricaVCardParamType *parray, 
					    RubricaVCardValueType *varray, 
					    gchar *data);

struct _RubricaVCardPrivate {
  gboolean dispose_has_run;
};



GType
rubrica_vcard_get_type()
{
  static GType rubrica_vcard_type = 0;
  
  if (!rubrica_vcard_type)
    {
      static const GTypeInfo rubrica_vcard_info =
	{
	  sizeof(RubricaVCardClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_vcard_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaVCard),
	  0,
	  (GInstanceInitFunc) rubrica_vcard_init
	};
      
      rubrica_vcard_type = g_type_register_static (G_TYPE_OBJECT,
						   "RubricaVCard",
						   &rubrica_vcard_info, 0);
    }
  
  return rubrica_vcard_type;
}



static void
rubrica_vcard_init(RubricaVCard* vcard)
{
  g_return_if_fail(RUBRICA_VCARD(vcard) != NULL);

  vcard->fp  = NULL;
  
  vcard->private = g_new(RubricaVCardPrivate, 1);
  vcard->private->dispose_has_run = FALSE;
}


static void
rubrica_vcard_class_init(RubricaVCardClass* klass)
{
  GObjectClass *class;
  
  class = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_vcard_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_vcard_finalize;

  /**
   * RubricaVCard::file-opened:
   * @vcard: the #RubricaVCard that receive the signal
   * @error: a #RubricaError code
   *
   * the "file-opened" signal is emitted when a vcard file has been 
   * opened. The error code is %RUBRICA_VCARD_FILE_OPEN_SUCCESS or 
   * %RUBRICA_VCARD_FILE_OPEN_FAIL
   */
  rubrica_vcard_signals[VCARD_FILE_OPENED] = 
    g_signal_new("file_opened", 
		 RUBRICA_VCARD_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaVCardClass, file_opened),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,         /* return type              */
		 1,                   /* params                   */
		 G_TYPE_INT);         /* params type - error code */

  /**
   * RubricaVCard::card-decoded
   * @vcard: the #RubricaVCard that receive the signal
   * @card: a gpointer
   *
   * the "card-decoded" signal is emitted when a card has been decoded from 
   * vcard file. To callback function is passed the decoded card.
   */
  rubrica_vcard_signals[VCARD_CARD_DECODED] = 
    g_signal_new("card_decoded", 
		 RUBRICA_VCARD_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaVCardClass, card_decoded),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);     /* the decoded card */

  /**
   * RubricaVCard::addressbook-decoded
   * @vcard: the #RubricaVCard that receive the signal
   *
   * this signal is emitted when vcard addressbook has been read
   */
  rubrica_vcard_signals[VCARD_ADDRESSBOOK_DECODED] =     
    g_signal_new("addressbook_decoded", 
		 RUBRICA_VCARD_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaVCardClass, addressbook_decoded),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);
}


static void 
rubrica_vcard_dispose (RubricaVCard* vcard)
{
  g_return_if_fail(IS_RUBRICA_VCARD(vcard));

  if (vcard->private->dispose_has_run)
    return;

  vcard->private->dispose_has_run = TRUE;
}


static void 
rubrica_vcard_finalize (RubricaVCard* vcard)
{
  g_return_if_fail(IS_RUBRICA_VCARD(vcard));
  
  g_free(vcard->private);
  vcard->private = NULL;
}


static gchar* 
rubrica_vcard_append_data(gchar* buf, gchar* data)
{
  if (buf)
    {
      gchar* tmp;

      tmp =  g_strdup_printf("%s%s", buf, data);

      g_free(buf);
      buf = tmp;
    }
  else
    buf = g_strdup(data);

  return buf;
}


static time_t 
rubrica_vcard_decode_date (gchar *date, gboolean has_hour)
{

  /* date: 2001-10-21T15:53:29  (tm)
           1973-03-18
	   123456789            (time_t)
  */
  struct tm tm;
  time_t t;
  gchar *str, *tok;

  if (!date) 
    return -1;

  /* test if date is a 'time' format (ex: 123456789) */
  str = date;
  for (; *str; str++)
    if (*str == '-')
      break;
    else
      return (time_t) atoi(date);
  
  str = g_strdup(date);
  tok = strtok(str, "-");
  tm.tm_year = atoi(tok) - 1900;
  tok = strtok(NULL, "-");
  tm.tm_mon = atoi(tok) - 1;
  tok = strtok(NULL, "T");
  tm.tm_mday = atoi(tok);
  

  if (has_hour)
    {
      tok = strtok(NULL, ":");
      tm.tm_hour = atoi(tok);
      tok = strtok(NULL, ":");
      tm.tm_min = atoi(tok);
      tok = strtok(NULL, "");
      tm.tm_sec = atoi(tok);
    }

  g_free(str);
  t = mktime(&tm);

  return t;
}


static  RubricaVCardTokenType 
rubrica_vcard_decode_token(gchar *string)
{
  gint i=0;
  
  g_return_val_if_fail(string != NULL, BAD_TOKEN);
    
  while (tokens[i].str)
    {
      if (string && ((strcasecmp(string, tokens[i].str)) == 0))
	return (RubricaVCardTokenType) tokens[i].tok;
      else
	i++;
    }

  return (RubricaVCardTokenType) BAD_TOKEN;
}


static RubricaVCardParamType 
rubrica_vcard_decode_param(gchar *string)
{
  gint i=0;
  
  g_return_val_if_fail(string != NULL, BAD_PARAM);
    
  while (paramtokens[i].str)
    {
      if (string && ((strcasecmp(string, paramtokens[i].str)) == 0))
	return (RubricaVCardParamType) paramtokens[i].tok;
      else
	i++;
    }

  return (RubricaVCardParamType) BAD_PARAM;
}


static RubricaVCardValueType 
rubrica_vcard_decode_value(gchar *string)
{
  gint i=0;
  
  g_return_val_if_fail(string != NULL, BAD_VALUE);
  
  while (valuetokens[i].str)
    {
      if (string && ((strcasecmp(string, valuetokens[i].str)) == 0))	
	return (RubricaVCardValueType) valuetokens[i].tok;
      else
	i++;
    }
  
  return (RubricaVCardValueType) BAD_VALUE;
}


static gchar* 
rubrica_vcard_extract_tokens(gchar* buffer,
			     RubricaVCardTokenType* token,
			     RubricaVCardParamType* param,
			     RubricaVCardValueType* value)
{
  gchar tok[BUFLEN];
  gchar dt[BUFLEN];
  gchar buf[BUFLEN];
  gboolean build_param = FALSE;
  gboolean build_value = FALSE;
  gboolean build_data  = FALSE;
  gint i = 0, j = 0, k = 0;

  memset(tok, 0, BUFLEN);
  memset(dt,  0, BUFLEN);
  memset(buf, 0, BUFLEN);

  param[0] = BAD_PARAM;
  value[0] = BAD_VALUE;

  /* build token */
  j = 0;
  while(buffer[i] && (buffer[i] != TOKEN_DELIM) && (buffer[i] != PARAM_DELIM))
    {
      if (buffer[i] == '\\' && ((buffer[i+1] == 'n') || (buffer[i+1] == 'r')))
	{
	  tok[j] = '\n';
	  i++;
	}
      else
	tok[j] = buffer[i];

      j++;	
      i++;      
    }
  *token = rubrica_vcard_decode_token(tok);

  /* build param */
  build_param = TRUE;
  if (buffer[i] == PARAM_DELIM)
    i++;

  j = 0;
  while(buffer[i] && !build_data) 
    {
      if (buffer[i] == TOKEN_DELIM)   // : il buffer rimanente contiene dati
	{
	  build_data = TRUE;

	  if (build_value)
	    value[k] = rubrica_vcard_decode_value(buf);
	  else if (j > 0)
	    {
	      param[k] = TYPE;
	      value[k] = rubrica_vcard_decode_value(buf);
	    }
	}
      else
	{ 
	  if (build_param && buffer[i] == VALUE_DELIM)    // = 
	    {
	      build_value = TRUE;
	      build_param = FALSE;

	      param[k] = rubrica_vcard_decode_param(buf);
	      memset(buf, 0, BUFLEN);
	      i++;
	      j = 0;
	    }
	  
	  if (build_value && buffer[i] == PARAM_DELIM)    // ;
	    {
	      build_param = TRUE;
	      build_value = FALSE;
	      
	      value[k] = rubrica_vcard_decode_value(buf);
	      memset(buf, 0, BUFLEN);
	      i++;
	      j = 0;
	      k++;
	    }
	  
	  if (build_param && buffer[i] == PARAM_DELIM)    // ;
	    {
	      param[k] = TYPE;
	      value[k] = rubrica_vcard_decode_value(buf);
	      memset(buf, 0, BUFLEN);
	      i++;
	      j = 0;
	      k++;
	    }  
	  
	  buf[j] = buffer[i];
	  i++;
	  j++; 
	}
    }
 
  /* read data */
  i++;
  j = 0;
  while(buffer[i])
    {
      if (buffer[i] == '\\' && ((buffer[i+1] == 'n') || (buffer[i+1] == 'r')))
	{
	  dt[j] = '\n';
	  i++;
	}
      else
	dt[j] = buffer[i];
      
      j++;      
      i++;
    }

  if (strcmp(dt, "") == 0)
    return NULL;
  
  return g_strdup(dt);
}


static void 
rubrica_vcard_route_tokens(RubricaVCard* vcard,
			   gchar* buffer, 
			   gchar* data, 
			   RubricaVCardTokenType token, 
			   RubricaVCardParamType* parray, 
			   RubricaVCardValueType* varray)
{  
  g_return_if_fail(IS_RUBRICA_VCARD(vcard));

  if ((token != BAD_TOKEN) && 
      ((oldtoken == NOTE)  || (oldtoken == ADR) || 
       (oldtoken == LABEL) || (oldtoken == CATEGORIES)))
    {
      multiline_data = FALSE;
      
      rubrica_vcard_build_card(vcard, oldtoken, oldparam, oldvalue, tmpbuf);

      oldtoken = BAD_TOKEN;
      g_free(tmpbuf);
      tmpbuf = NULL;
    }
  
  if (token == NOTE)
    {
      oldtoken  = NOTE;
      *oldparam = BAD_PARAM;
      *oldvalue = BAD_VALUE;
      
      multiline_data = TRUE;
      if (tmpbuf)
	g_free(tmpbuf);

      tmpbuf = rubrica_vcard_append_data(NULL, data);
    }
  
  if (token == ADR)
    {
      memset(oldparam, 0, ARRAY_LEN);
      memset(oldvalue, 0, ARRAY_LEN);
      
      oldtoken  = ADR;
      *oldparam = *parray;
      *oldvalue = *varray;	      
      
      multiline_data = TRUE;
      if (tmpbuf)
	g_free(tmpbuf);
      
      tmpbuf = rubrica_vcard_append_data(NULL, data);      
    }
  
  if (token == LABEL)
    {
      memset(oldparam, 0, ARRAY_LEN);
      memset(oldvalue, 0, ARRAY_LEN);
      
      oldtoken  = LABEL;
      *oldparam = *parray;
      *oldvalue = *varray;	      
      
      multiline_data = TRUE;
      if (tmpbuf)
	g_free(tmpbuf);
      
      tmpbuf = rubrica_vcard_append_data(NULL, data); 
    }
  
  if (token == CATEGORIES)
    {
      memset(oldparam, 0, ARRAY_LEN);
      memset(oldvalue, 0, ARRAY_LEN); 
      
      oldtoken  = CATEGORIES; 
      *oldparam = BAD_PARAM; 
      *oldvalue = BAD_VALUE;       
      
      multiline_data = TRUE;
      if (tmpbuf) 
	g_free(tmpbuf); 
      
      tmpbuf = rubrica_vcard_append_data(NULL, data); 
    }

  if (buffer && (token == BAD_TOKEN))
    tmpbuf = rubrica_vcard_append_data(tmpbuf, buffer);
  
  if (!multiline_data)
    rubrica_vcard_build_card(vcard, token, parray, varray, data);     
  
  if (data)
    g_free(data);	      
}


static RubricaCompanyCard* 
rubrica_vcard_company_new(RubricaCard* card)
{
  RubricaRef* ref;
  RubricaCompanyCard* company = NULL;
  gchar* id;
  gchar* info;
      
  have_company = TRUE;
  id  = rubrica_card_get_id(RUBRICA_CARD(card));
  ref = rubrica_ref_new(id);

  g_object_get(RUBRICA_CARD(card), "card-name", &info, NULL);
  g_object_set(ref, "ref-info", info, NULL);

  company = rubrica_company_card_new();
  rubrica_company_card_add_ref(company, ref);	  
  
  return company;
}


static RubricaAddress*
rubrica_vcard_company_address_new(void)
{
  have_company_address = TRUE;
  return rubrica_address_new();
}


static gint 
rubrica_vcard_split_data(gchar* data)
{
  gchar *tmp, buffer[128];
  gint j = 0;
  gint i = 0;

  memset(buffer, 0, 128);
  
  tmp = data;
  for (; *tmp; tmp++)
    {
      if (*tmp != ';')
	buffer[j++] = *tmp;
      else
	{
	  buffer[j] = '\0';
	  decoded[i] = g_strdup(buffer);
	  i++;
	  j = 0;
	}
    }
 
  return i;
}


static void 
rubrica_vcard_build_card(RubricaVCard *vcard, 
			 RubricaVCardTokenType token,
			 RubricaVCardParamType *parray, 
			 RubricaVCardValueType *varray, 
			 gchar *data)
{
  RubricaAddressType addtype;
  time_t time;
  gint i=0;
  gchar** categories = NULL;

  g_return_if_fail(IS_RUBRICA_VCARD(vcard));
  
  if (!data) return;

  switch(token)
    {
    case BEGIN:
      card    = rubrica_personal_card_new();
      contact = rubrica_contact_new();
      work    = rubrica_work_new();
      notes   = rubrica_notes_new();
      break;
      
    case END:
      rubrica_personal_card_set_contact(card, contact);
      rubrica_personal_card_set_notes(card, notes);
      rubrica_personal_card_set_work(card, work);

      g_signal_emit_by_name(vcard, "card_decoded", card, G_TYPE_POINTER);

      if (have_company)
	{
	  gchar *name;

	  /* set card name to company name 
	   */
	  g_object_get(company, "company-name", &name, NULL);
	  g_object_set(RUBRICA_CARD(company), "card-name", name, NULL);

	  rubrica_card_add_address(RUBRICA_CARD(company), company_address);
	  have_company = FALSE;
	  have_company_address = FALSE;

	  g_signal_emit_by_name(vcard,"card_decoded", company, G_TYPE_POINTER);
	}
      break;

    case FN:
      /* card name */ 
      g_object_set(RUBRICA_CARD(card), "card-name", data, NULL);
      break;
      
    case N:
      /* ord:: surname (last name), (given) name, (middle) second name,
	       prefix, suffix (title) 
	       cognome, nome, secondo nome, prefisso, suffisso (titolo) 
      */
      rubrica_vcard_split_data(data);

      g_object_set(contact, "last-name", decoded[0],
		   "first-name", decoded[1], "middle-name", decoded[2],
		   "prefix", decoded[3], "title", decoded[4], NULL);

      for (i = 0; i <8; i++)
	if (decoded[i])
	  {
	    g_free(decoded[i]);
	    decoded[i] = NULL;
	  }
      break;

    case TZ:
      break;

    case REV:
      break;

    case ADR:
      /* mail address + mail type 
	 indirizzo di posta + tipo 

	 pbox ==> decoded[0], extadd ==> decoded[1] 
	 
	 state = province = decoded[4]
      */

      address = rubrica_address_new();
      rubrica_vcard_split_data(data);

      g_object_set(address, "street", decoded[2], "city", decoded[3],
		   "zip", decoded[5], "country", decoded[6],
		   "state", decoded[4], "province", decoded[4], NULL);
        
      for (i = 0; i <8; i++)
	if (decoded[i])
	  {
	    g_free(decoded[i]);
	    decoded[i] = NULL;
	  }
     
      switch(*varray)
	{
	case HOME:
	  addtype = RUBRICA_ADDRESS_HOME;
	  break;

	case WORK:
	  addtype = RUBRICA_ADDRESS_WORK;
	  break;

	case BAD_VALUE:
	  addtype = RUBRICA_ADDRESS_INVALID;
	  break;

	default:
	  break;	      
	} 

      g_object_set(address, "address-type", addtype, NULL);
      rubrica_card_add_address(RUBRICA_CARD(card), address);
      break;
      
    case TEL:
      /* telefono + tipo */   
      if (*varray)
	{
	  RubricaVCardValueType value;
	  RubricaTelephoneType type;
	  gboolean unknown;
		  
	  do {
	    unknown = FALSE;
	    value = (RubricaVCardValueType) *varray;
	    
	    switch (value)
	      {	  
	      case HOME:
		type = RUBRICA_TELEPHONE_HOME;
		break;

	      case WORK:
		type = RUBRICA_TELEPHONE_WORK;
		break;

	      case FAX:
		type = RUBRICA_TELEPHONE_FAX;
		break;

	      case CELL:
		type = RUBRICA_TELEPHONE_CELLPHONE;
		break;
		
	      case BAD_VALUE:
		type = RUBRICA_TELEPHONE_UNKNOWN;
		unknown = TRUE;
		break;
		
		/* other telephone type */
	      case PREF:
	      case VOICE:
	      case MSG:
	      case PAGER:
	      case BBS:
	      case MODEM:
	      case CAR:
	      case ISDN:
	      case VIDEO:
	      default:
		type = RUBRICA_TELEPHONE_OTHER;
		unknown = TRUE;				  
		break;
	      }
	    
	    tel = rubrica_telephone_new();
	    g_object_set(tel, "telephone-number", data, 
			 "telephone-type", type, NULL);
	    
	    rubrica_card_add_telephone(RUBRICA_CARD(card), tel);
	    varray++;
	  } while (*varray && unknown == TRUE);
	}
      break;
      
    case EMAIL:  
      net = rubrica_net_address_new();
      
      g_object_set(net, "url", data, 
		   "url-type", RUBRICA_NET_ADDRESS_EMAIL, NULL);
      rubrica_card_add_net_address(RUBRICA_CARD(card), net);
      break;

    case URL:      
      net = rubrica_net_address_new();
      
      g_object_set(net, "url", data, 
		   "url-type", RUBRICA_NET_ADDRESS_WEB, NULL);
      rubrica_card_add_net_address(RUBRICA_CARD(card), net);
     break;

    case GEO:
      break;

    case BDAY:
      time = rubrica_vcard_decode_date(data, FALSE);
      //      rubrica_contact_set_birthday(contact, time);
      break;
      
    case TITLE:
    case KDE_EXT_Profession:
      g_object_set(contact, "profession", data, NULL);
      break;

    case ORG:
      g_object_set(work, "organization", data, NULL);
      break;

    case CATEGORIES:
    case RUBRICA_EXT_Group:
      categories = g_strsplit(data, ",", 0);
      for (i = 0; categories[i]; i++)
	{
	  RubricaGroup* group;

	  group = rubrica_group_new();
	  g_object_set(group, "group-name", categories[i], 
		       "group-owner", "user", NULL);

	  /*
	  rubrica_group_set_name(group, categories[i]);
	  rubrica_group_set_owner(group, "user");
	  rubrica_card_add_group(RUBRICA_CARD(card), group); 
	  */
	}
      g_strfreev(categories); 
      break;
      
    case NOTE:
      g_object_set(notes, "other-notes", data, NULL);
      break;
      
    case KEY: 
      g_object_set(notes, "pubkey", data, NULL);
      break;

    case ROLE:
      g_object_set(work, "assignment", data, NULL);
      break;

    case UID:
      break;

    case LOGO:
      break;

    case PHOTO:
      break;

    case LABEL:
      break;

    case AGENT:
      break;

    case PROID:
      break;

    case SOUND:
      break;

    case CLASS:
      break;

    case MAILER:
      break;

    case VERSION:
      break;

    case NICKNAME:
      g_object_set(contact, "nick-name", data, NULL);
      break;
      
    case RUBRICA_EXT_Department:
    case KDE_EXT_Department:
      g_object_set(work, "department", data, NULL);
      break;
      
    case RUBRICA_EXT_SubDepartment:
      g_object_set(work, "sub-department", data, NULL);
      break;

    case RUBRICA_EXT_CollaboratorName:
    case KDE_EXT_AssistantsName:
    case EVOLUTION_EXT_Assistant:
      g_object_set(work, "collaborator", data, NULL);
      break;
      
    case RUBRICA_EXT_CollaboratorPhone:
      g_object_set(work, "collaborator-phone", data, NULL);
      break;
      
    case RUBRICA_EXT_SpouseName:
    case KDE_EXT_SpousesName:
    case EVOLUTION_EXT_Spouse:
      g_object_set(notes, "has-partner", TRUE, "partner-name", data, NULL);
      break;
      
    case RUBRICA_EXT_Child:
      g_object_set(notes, "children", data, NULL);
      break;

    case RUBRICA_EXT_Anniversary:
    case KDE_EXT_Anniversary:
    case EVOLUTION_EXT_Anniversary:
      time = rubrica_vcard_decode_date(data, FALSE);
      rubrica_notes_set_know_anniversary(notes, TRUE);
      //      rubrica_notes_set_anniversary(notes, time);
    break;
      
    case RUBRICA_EXT_ManagerName:
    case KDE_EXT_ManagersName:
    case EVOLUTION_EXT_Manager:
      g_object_set(work, "manager-name", data, NULL);
      break;

    case KDE_EXT_Office:
    case EVOLUTION_EXT_Office:
      break;

    case KDE_EXT_CUSTOM:  
      break;

      /* Company tokens 
       */
    case RUBRICA_EXT_CompanyName:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      g_object_set(company, "company-name", data, NULL); 
      g_object_set(RUBRICA_CARD(company), "card-name", data, NULL);
      break;

    case RUBRICA_EXT_Street:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      if (!have_company_address)
	company_address = rubrica_vcard_company_address_new();

      g_object_set(company_address, "street", data, NULL);
      break;
      
    case RUBRICA_EXT_StreetNumber:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      if (!have_company_address)
	company_address = rubrica_vcard_company_address_new();

      g_object_set(company_address, "street-number", data, NULL);
      break;
      
    case RUBRICA_EXT_ZipCode:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      if (!have_company_address)
	company_address = rubrica_vcard_company_address_new();

      g_object_set(company_address, "zip", data, NULL);
      break;
      
    case RUBRICA_EXT_City:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      if (!have_company_address)
	company_address = rubrica_vcard_company_address_new();

      g_object_set(company_address, "city", data, NULL);
      break;
      
    case RUBRICA_EXT_Province:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      if (!have_company_address)
	company_address = rubrica_vcard_company_address_new();

      g_object_set(company_address, "province", data, NULL);
      break;
      
    case RUBRICA_EXT_Country:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      if (!have_company_address)
	company_address = rubrica_vcard_company_address_new();

      g_object_set(company_address, "country", data, NULL);
      break;
      
    case RUBRICA_EXT_Web:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      company_net = rubrica_net_address_new();
      g_object_set(company_net, "url", data, 
		   "url-type", RUBRICA_NET_ADDRESS_WEB, NULL);
      rubrica_card_add_net_address(RUBRICA_CARD(company), company_net);
      break;
      
    case RUBRICA_EXT_Email:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      company_net = rubrica_net_address_new();
      g_object_set(company_net, "url", data, 
		   "url-type", RUBRICA_NET_ADDRESS_EMAIL, NULL);
      rubrica_card_add_net_address(RUBRICA_CARD(company), company_net);
      break;
      
    case RUBRICA_EXT_Operator:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      company_telephone = rubrica_telephone_new();
      g_object_set(company_telephone, "telephone-number", data, 
		   "telephone-type", RUBRICA_TELEPHONE_OPERATOR, NULL);
      rubrica_card_add_telephone(RUBRICA_CARD(company), company_telephone);
      break;

    case RUBRICA_EXT_Fax:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      company_telephone = rubrica_telephone_new();
      g_object_set(company_telephone, "telephone-number", data, 
		   "telephone-type", RUBRICA_TELEPHONE_FAX, NULL);
      rubrica_card_add_telephone(RUBRICA_CARD(company), company_telephone);
      break;
      
    case RUBRICA_EXT_Green:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      company_telephone = rubrica_telephone_new();
      g_object_set(company_telephone, "telephone-number", data, 
		   "telephone-type", RUBRICA_TELEPHONE_GREEN, NULL);      
      rubrica_card_add_telephone(RUBRICA_CARD(company), company_telephone);
      break;
      
    case RUBRICA_EXT_CustomerCare:
      if (!have_company) 
	company = rubrica_vcard_company_new(RUBRICA_CARD(card));
      
      company_telephone = rubrica_telephone_new();
      g_object_set(company_telephone, "telephone-number", data, 
		   "telephone-type", RUBRICA_TELEPHONE_CUSTOMER_CARE, NULL);
      rubrica_card_add_telephone(RUBRICA_CARD(company), company_telephone);
      break;

    case EVOLUTION_EXT_File_as:
      break;

      
    case BAD_TOKEN:      
    default:
      break;
    }
}





/* Public functions
 */
/**
 * rubrica_vcard_new
 * 
 * create a new #RubricaVCard
 *
 * returns: a #RubricaVCard*
 */
RubricaVCard* 
rubrica_vcard_new(void)
{
  RubricaVCard *vcard;

  vcard = g_object_new(rubrica_vcard_get_type(), NULL);

  return (RubricaVCard*) vcard;
}


/**
 * rubrica_vcard_free
 * @vcard: a #RubricaVCard
 *
 * free the #RubricaVCard
 */
void 
rubrica_vcard_free_vcard(RubricaVCard *vcard)
{
  g_return_if_fail(IS_RUBRICA_VCARD(vcard));

  g_object_unref(vcard);
}


/**
 * rubrica_vcard_open_file
 * @vcard: a #RubricaVCard
 * @filename: the vcard file to read
 *
 * Open/read a vcard addressbook. The vcard addressbook is opened, 
 * for each card read successfully, the card-decoded signal is emitted
 * and a pointer to decoded card is sent to the callback function. 
 * #RubricaAbookVCard use rubrica_vcard_open_file() to perform the 
 * addressbook load.
 *
 * returns: a #RubricaError  
 */
RubricaError 
rubrica_vcard_open_file (RubricaVCard* vcard, gchar* filename)
{
  gchar buffer[BUFLEN]; 
  gunichar ch;
  gchar* data;
  GString *string;
  gint i;

  RubricaVCardTokenType token;
  RubricaVCardParamType parray[ARRAY_LEN];
  RubricaVCardValueType varray[ARRAY_LEN];

  g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), -1);
  g_return_val_if_fail(filename != NULL, -1);

  vcard->fp = fopen(filename, "r");
  if (!vcard->fp)
    {
      g_signal_emit_by_name(vcard, "file_opened", 
			    RUBRICA_VCARD_FILE_OPEN_FAIL, G_TYPE_INT);

      return RUBRICA_VCARD_FILE_OPEN_FAIL;
    }
  else
    g_signal_emit_by_name(vcard, "file_opened",
			  RUBRICA_VCARD_FILE_OPEN_SUCCESS, G_TYPE_INT);

  memset(buffer, 0, BUFLEN);
  memset(parray, 0, ARRAY_LEN);
  memset(varray, 0, ARRAY_LEN);
  string = g_string_new(NULL);

  do
    {
      ch = fgetc(vcard->fp);
      
      if ((ch != feof(vcard->fp)) && (ch != '\n') && (ch != '\r'))
	string = g_string_append_unichar(string, ch); 
      else if ((ch != feof(vcard->fp)))
	{
	  if (g_utf8_validate(string->str, -1, NULL))
	    g_utf8_strncpy(buffer, string->str, g_utf8_strlen(string->str,-1));
	  
	  if (buffer)
	    {
	      data = rubrica_vcard_extract_tokens(buffer, &token, 
						  parray, varray);
	      
	      rubrica_vcard_route_tokens(vcard, buffer, data,
					 token, parray, varray);	      
	    }
	  
	  g_string_free(string, TRUE);
	  memset(buffer, 0, BUFLEN);
	  for (i = 0; i < ARRAY_LEN; i++) parray[i] = varray[i] = 0;
	  string = g_string_new(NULL);	  
	}
    }
  while(!feof(vcard->fp));

  g_signal_emit_by_name(vcard, "addressbook_decoded", NULL, G_TYPE_NONE);

  fclose(vcard->fp);
  g_string_free(string, TRUE);
  
  return RUBRICA_VCARD_FILE_IMPORT_SUCCESS;
}

